# -*- coding: utf-8 -*-

# Copyright 2006-2012 Borislav Rizov <bobyATdclDOTbasDOTbg>, Department of Computational Linguistic, Institute for Bulgarian Language – Bulgarian Academy of Sciences
# 
# This file is part of Hydra.
# 
# Hydra is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hydra is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Hydra.  If not, see <http://www.gnu.org/licenses/>.

from calc import Processor
from db import *
from descriptor import *

class WN(DB):
    """
        Предлага интерфейс към набор от интерфейси за различни, но свързани задачи.
        -като наследник на DB, предоставя методи за достъп до базата от данни.
        -метод за извличането на множеството на лингвистичните единици определено от дадена модална формула.
        -
        -предостава функции за обработка на заключванията в системата
    """
    
    def __init__(self):
        DB.__init__(self)
        self.processor = Processor()
        self.pool = CachedTablePool(self)
        self.pos = DCachedTable(self, POSId)
    
    def create_langs(self):
        _langs = self.get_all(LangId.table)
        langs = [Lang(l) for l in _langs]
        return dict([[lang['id'], lang] for lang in langs])
    
    def add_lang(self, lang_name):
        for i,  lang in self.langs.iteritems():
            if  lang['name'] == lang_name:
                return lang
        self.insert(LangId.table, (0, lang_name, 0))
        _id = self.last_id()
        lang = Lang(self.get_one(_id, LangId.table))
        self.langs = self.create_langs()
        return lang
        #self.langs = self.create_langs()

    def get(self, formula):
        return map(self.ling,  self.get_ids(formula))
    
    def get_ids(self, formula):
        query = self.processor.process(formula)
#        print query
        #print type(query), "get"
        cursor = self.execute(query)
        
        ids = cursor.fetchall()
        return  [id[0] for id in ids]
    
    def get_rel(self, _rel, ling):
#		print 'get_rel',  _rel
        return self.get(u"<%s>%s" % (_rel, ling.ID()))
    
    def encode(self, ili, lang, new = 0):
#        print 'encode', ili, lang

        m = re.search(r"-([0-9]*)L?(-.)?$", ili)
#        m = re.search(r"-([0-9]*)L?.*$", ili)
#        print ili
        if not m: 
            print '-------------',  ili
#        print m.group(1),  ili
#        m = re.search(r"-([0-9]*)", ili)
        #if not m:
            #print ili
        code = `int(lang['id'])` + `new` + m.group(1)
        n = re.search(r"-(.)$", ili)
        if n:
            code += str(self.pos.failsafe_add(n.group(1)))
        else:
            code += '0'
        return  long(code)
    def decode(self, synset):
        return self.get_one(synset.ID(), SynsetId.table)[SynsetId.ili]

    def key(self, _id):
        row = self.get_one(_id, LockId.table)
        if row:
            return list(row)
        else:
            return None
    def locked(self, _id):
        row = self.key(_id)
        if row:
            return row[LockId.user] != self.user
        else:
            return False
    def lock(self, _id):
        row = self.key(_id)
        if row:
            row[LockId.value] += 1
            self.replace(LockId.table, row)
        else:
            self.replace(LockId.table, (_id, None, self.user, 1))
    def unlock(self, _id):
        row = self.key(_id)
        if row:
            if row[LockId.value] > 1:
                row[LockId.value] -= 1
                self.replace(LockId.table, row)
            else:
                self.execute(u"delete from %s where id = %d" % (LockId.table, _id))
    
    def forward_relations(self):
        from relations import ReverseBase
        for rel in self.relations.values():
            if not isinstance(rel, ReverseBase):
                yield rel
    
    def new_sense(self, word):
        cursor = self.execute("select max(sense) from Literal where word = '%s'" % word)
        sense = cursor.fetchone()[0]
        if sense:
            return sense + 1
        return 1

class Lang:
    def __init__(self, info):
        self.info = list(info)
    def __getitem__(self, item): #Fixme: make Lang and Relation work in the same manner
        return self.info[LangId[item]]
    def ID(self):
        return self['id']
    def make_ili(self):
        sid = long(self.info[LangId.synset_id])
        self.info[LangId.synset_id] = sid + 1
        wn.replace(LangId.table, self.info)
        return self['name'] + "-" + "0"*(8 - len(`sid`)) + `sid`
global wn
wn = WN()

