# -*- coding: utf-8 -*-

# Copyright 2006-2012 Borislav Rizov <bobyATdclDOTbasDOTbg>, Department of Computational Linguistic, Institute for Bulgarian Language – Bulgarian Academy of Sciences
# 
# This file is part of Hydra.
# 
# Hydra is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hydra is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Hydra.  If not, see <http://www.gnu.org/licenses/>.

from feeder import Feeder,  XmlConfiger,  encode_data,  decode_data
from observer_context import  *
from common import *
from Tix import *
from wordnet import *
import os

class LingViewMemento:
    def __init__(self, path, ling, mode):
        self.path = path
        self.ling = ling
        self.mode = mode
        

class LingView(Frame):
    def __init__(self, dict, parent):
        Frame.__init__(self, parent)
        self.dict = dict
        self.create_widgets()
        self.ling = None
        self.path = ''
        configer =  XmlConfiger(os.path.join(os.path.dirname(__file__) , "unit_view.xml"))
        self.feeders = configer.feeders
        self.high = None
    
    def lang(self):
        return self.dict.lang
    
    def history(self):
        return self.dict.history()
    
    def create_widgets(self):
        self.label = Label(self)
        self.label.pack(anchor = 'w')
        # Tree
        self.tree = Tree(self, opencmd = self.opencmd, closecmd = self.closecmd)
        self.tree.hlist.configure(selectforeground = 'black')
        self.tree.pack(fill = BOTH, expand = 1)
    
    def sel_ling(self, path):
        if self.exists(path):
            data = self.tree.hlist.info_data(path)
            sid = decode_data(data)[0]
            ling = wn.ling(long(sid))
            return ling
        return None
    
    def set_ling(self, ling, path = ''):
        self.path = path
        self.ling = ling
        self.refresh_label()
    
    def find_main(self, path):
        parent = self.parent(path)
        if parent and self.sel_ling(path) == self.sel_ling(parent):
            return parent
        return path
    
    def enable_path(self, path, mode):
        if path != "":
            self.tree.setmode(path, mode)
            parent = self.tree.hlist.info_parent(path)
            self.enable_path(parent, mode)
    
    def open(self, ling, path):
        self.feed(ling, path)
        self.tree.setmode(path, 'close')
    
    def close(self, path):
        self.tree.hlist.delete_offsprings(path)
        self.tree.setmode(path, 'open')
    
    def opencmd(self, path):
        self.open(self.sel_ling(path), path)
    
    def closecmd(self, path):
        self.close(path)
    
    def invoke(self, path, mode):
        if mode == 'close':
            self.open(self.ling, path)
        elif mode == 'open':
            self.close(path)
        else:
            self.feed(self.ling, path)
        self.path = path
    
    def clear(self):
        self.tree.hlist.delete_all()
        self.path = ''
        self.ling = None
        self.label["text"] = ''
    
    def feed(self, ling, path):
        name = class_name(ling)
        feeder = self.feeders[name]
        self.tree.hlist.delete_offsprings(path)
        feeder.feed(ling, self, path)
    
    def redraw(self, mode):
        path = self.find_main(self.path)
        self.refeed(self.ling, path)
        if mode == 'none':
            self.enable_path(path, mode)
        elif self.tree.getmode(path) == 'none':
            pmode = 'close'
            self.enable_path(self.tree.hlist.info_parent(path), pmode)
        self.invoke(path, mode)
        self.refresh_label()
        
    def enabled(self):
        return self.ling == None or not self.ling.editable()
    
    def refresh(self, message):
        self._refresh(message.data, self.get_mode(message))
    
    def find_child(self, ling, parent):
        path = self.find_main(parent)
        for child in self.tree.hlist.info_children(path):
            if self.sel_ling(child).ID() == ling.ID():
                return child
        return None
    
    def select(self, path):
        if path != self.path:
            self.set_ling(self.sel_ling(path), path)
    
    def memento(self):
        if self.path == '':
            mode = None
        else:
            mode = self.tree.getmode(self.path)
        return LingViewMemento(self.path, self.ling, mode)
    
    def set_memento(self, memento):
        if memento.ling == None:
            self.clear()
        else:
            if self.exists(memento.path):
                self.set_ling(memento.ling, memento.path)
                self.redraw(memento.mode)
            else:
                self._refresh(memento.ling, memento.mode)
    
    def refeed(self, ling, path):
        if self.high:
            self.high.highlight(False)
        
        if self.tree.hlist.info_parent(path) == '':
            self.tree.hlist.entryconfigure(path, text = class_name(ling) + ": " + ling.to_string(), data = encode_data(ling.ID(), -1, ''))
        else:
            parent = self.tree.hlist.info_parent(path)
            name = class_name(self.sel_ling(parent))
            feeder = self.feeders[name]
#            if self.high: feeder.refeed(
            self.high = feeder.refeed(ling, self, path)
            self.high.highlight(True)

    def clear_feed(self, ling, mode):
        self.clear()
        path = self.tree.hlist.add_child('')
        if ling:
            synset = ling.synset()
            self.refeed(synset, path)
            self.feed(synset, path)
            self.tree.setmode(path, mode)
            if ling != synset:
                path = self.find_child(ling, path)
                self.feed(ling, path)
                self.tree.setmode(path, mode)
                return path
        return path
    
    def parent(self, path):
        if path == "":
            return None
        parent = self.tree.hlist.info_parent(path)
        if parent != "":
            return parent
        return None
    
    def get_mode(self, message):
        if message.data and message.data.editable():
            return 'none'
        if message.type & VM.Close:
            return 'open'
        return 'close'
    
    def _refresh(self, ling, mode):
        print mode
        if not ling:
            self.clear()
            return
        child = self.find_child(ling, self.path)
        parent = self.parent(self.path)
        path = None
        if self.ling and ling.ID() == self.ling.ID():
            path = self.path
        elif parent and self.sel_ling(parent).ID() == ling.ID():
            path = parent
        elif child:
            path = child
        if path:
            self.set_ling(ling, path)
            self.redraw(mode)
        else:
            path = self.clear_feed(ling, mode)
            self.set_ling(ling, path)
    
    def exists(self, path):
        return int(self.tree.hlist.info_exists(path))
    
    def refresh_label(self):
        if self.ling:
            text = class_name(self.ling) + ": " + self.ling.to_string()
        else:
            text = ""
        self.label['text'] = text

class EditLingView(LingView):
    def __init__(self, parent, dict):
        LingView.__init__(self, parent, dict)
    
    def opencmd(self, path):
        current_path = self.path
        rel = wn.relations.get(decode_data(self.tree.hlist.info_data(path))[2], None)
        if self.ling.editable():
            if path in self.tree.hlist.info_children(current_path) and rel and  rel.is_(RelChars.NewCreational):
            #path is child of current_path
                self.dict.open_child(self.sel_ling(path))
            else:
                self.open(self.sel_ling(path), path)
        else:
            self.select(path)
            self.dict.refresh(VMessage(self.sel_ling(path), VM.Select|VM.Open))
    
    def closecmd(self, path):
        if self.ling.editable():
            self.close(path)
        else:
            self.select(path)
            self.dict.refresh(VMessage(self.sel_ling(path), VM.Select|VM.Close))
    
