# -*- coding: utf-8 -*-

# Copyright 2006-2012 Borislav Rizov <bobyATdclDOTbasDOTbg>, Department of Computational Linguistic, Institute for Bulgarian Language – Bulgarian Academy of Sciences
# 
# This file is part of Hydra.
# 
# Hydra is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hydra is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Hydra.  If not, see <http://www.gnu.org/licenses/>.

from wordnet import *
import sys,  os,  codecs
import logging


class Map:
    mappings = ['mapping-20-30/wn20-30.a',  'mapping-20-30/wn20-30.b',  'mapping-20-30/wn20-30.n',  'mapping-20-30/wn20-30.v' ]
    fixes = 'mapping-20-30/wn20-30.fix'
    template='eng-30-%s'
    def __init__(self,  filenames=None,  fixes = None):
        self.back = {}
        self.map = {}
        self.conflicts = 0
        if not filenames:
            filenames = [os.path.join(os.path.dirname(__file__),  m) for m in self.mappings]
#            print filenames
        if not fixes:
            fixes = os.path.join(os.path.dirname(__file__), self.fixes)
#            print fixes
        self.load_fix(fixes)
        for filename in filenames:
            self.load(filename)
#        self.map.update(self.fix_map)
    def store_back(self,  src,  dest):
        for d in dest:
            if d in self.back:
#                print 'conflict',  self.back[d],  (src,  dest)
                self.conflicts += 1
            else:
                self.back[d] = (src,  dest)
    def add_fix(self,  s,  d):
        if 'remove' in d:
            self.fix_map[s] = 'r'
        else:
            self.fix_map[s] = d

    def add_fixes(self,  line,  pattern,  remove = True):
        m = re.match(pattern,  line)
        if m and '>>' in m.group(2):
#                print 'dest:',  m.group(1)
            if remove: self.remove[m.group(1)] = 1
            map = m.group(2).split(';')
            for m in map:
                if '>>' in m:
                    try:
                        s,  d = m.split('>>')
                    except:
                        logging.warning(m)
                    s = s.strip()
                    d = d.split()[0].strip()
                    self.add_fix(s,  d)

    def load_fix(self,  filename):
        self.fix_map = {}
        self.remove = {}
        file = codecs.open(filename,  'rt',  'cp1251')
        for line in file:
            if line.startswith('splits'):
                break
            self.add_fixes(line,  ur'(\d+\-.)\s+(.*\S.*)')
        for line in file:
            self.add_fixes(line,  ur'(\d+)\s+(.*\S.*)',  False)

    def store(self,  src,  dest):
        new_dest = []
        for d in dest:
            if d[0] not in self.remove:
                new_dest.append(d)
        if new_dest != dest:
            print 'diff',  new_dest,  dest
        self.map[src] = new_dest
    def load(self,  filename):
        pos = filename.split('.')[1]
        file = open(filename)
        for line in file:
            ids = line.split()
            dest = [(ids[i] + '-' + pos,  float(ids[i+1])) for i in range(1,  len(ids),  2)]
            dest.sort(key=lambda x: -x[1])
            src = ids[0] +'-' + pos
            self.store(src,  dest)
            self.store_back(src,  dest)
    def ili(self,  old_id):
        if old_id in self.fix_map:
            fix = self.fix_map[old_id]
            if fix != 'r':
                return [fix]
            else:
                return [old_id]
        version,  offset_pos = old_id.split('-',  1)
        if version == 'ENG20':
            new_offset = self.map.get(offset_pos,  None)
            if new_offset:
                return [self.template % x[0] for x in new_offset]
        return [old_id]

class BackMap(Map):
    mappings = ['mapping-30-20/wn30-20.a',  'mapping-30-20/wn30-20.b',  'mapping-30-20/wn30-20.n',  'mapping-30-20/wn30-20.v' ]
    template='ENG20-%s'
    def ili(self,  old_id):
        try:
            lang,  version,  offset_pos = old_id.split('-',  2)
            
            if lang == 'eng' and version == '30':
                new_offset = self.map.get(offset_pos,  None)
                if new_offset:
                    return [self.template % x[0] for x in new_offset]
            return [old_id]
        except:
            print 'old: %s|' % old_id
            raise
    

class Export:
    
    def __init__(self,  filenames=None):
        self.map = Map(filenames)
    def export_lang(self, lang, file):
        return self.export_query("lang('%s')" % lang)
    def export_query(self,  query,  file):
        #print len(synsets)
        synsets = wn.get(query)
        for synset in synsets:
            data = self.export_synset(synset)
            print >> file, data.encode('utf8')
            #return
    def export_ilis(self,  ilis,  file):
        query = '|'.join(["ili('%s')" % ili for ili in ilis])
        return self.export_query(query,  file)
    def tag(self, name, value,  attibs = []):
        atts = []
        return "<%s>%s</%s>" % (name, unicode(value).strip().replace('\n',  ' '), name)
    def export_literal(self, literal):
        word = literal["word"]
        sense = self.tag("SENSE", literal["sense"])
        lemma = self.tag("LEMMA", literal["lemma"])
        
        notes = self.export_notes(literal)
        return self.tag("LITERAL", word + sense + lemma + notes)
    def export_note(self, rel, ling):
        notes = rel.neighbours(ling)
        return "".join([self.tag(rel["name"].upper(), note["note"]) for note in notes])
    def export_notes(self, ling):
        notes = ""
        for rel in wn.forward_relations():
            if rel["id2"] is Note:
                notes += self.export_note(rel, ling)
        return notes
    def prepare_ilr(self,  rel,  ili):
        return self.tag("ILR",  ili+ self.tag("TYPE", rel))
    def export_ilr(self, rel, synset):
        synsets = rel.neighbours(synset)
        return "".join([self.prepare_ilr(rel["name"],  self.map.ili(syn["ili"])[0]) for syn in synsets])
    def export_ilrs(self, synset,  ilis):
        ilrs = ""
        for ili in ilis:
            ilrs += self.prepare_ilr("SPLIT",  ili)
        for rel in wn.forward_relations():
            if rel["id2"] is Synset:
                ilrs += self.export_ilr(rel, synset)
        return ilrs
    def export_synset(self, synset):
        ilis = self.map.ili(synset["ili"])
        ili = self.tag("ID", ilis[0])
        pos = self.tag("POS", synset["pos"])
        defin = self.tag("DEF", synset["definition"])
        stamp = self.tag("STAMP", synset["stamp"])
        bcs = self.tag("BCS", synset["bcs"])
        frequency = self.tag("FREQUENCY",  synset["frequency"])
        #literals
        syns = ""
        for literal in synset.literals():
            syns += self.export_literal(literal)
        synonyms = self.tag("SYNONYM", syns)
        
        #ilrs, notes
        ilrs = self.export_ilrs(synset,  ilis[1:])
        notes = self.export_notes(synset)
        
        result = self.tag("SYNSET", ili + pos + synonyms + defin + ilrs + stamp + bcs + notes+frequency)
        return result

if __name__ == '__main__':
    logging.basicConfig()
    def counts(map):
        count = 0
        all = 0
        for x in map.map:
            if len(map.map[x]) > 1:
                all += len(map.map[x])
                count += 1
        print count
        print float(all)/count
    def test():
        map = Map()
        bmap = BackMap()
        y = 'ENG20-04061777-n'
        x = map.ili(y)
        print y
        print x
        z = bmap.ili(x[0])
        print y == z[0]
        print counts(bmap)
    
    def test2():
        map = Map()
        print  map.ili('ENG20-08677230-n')
        print  map.ili('ENG20-08164451-n')
        print map.fix_map
    
    def main():
        export = Export()
        export.export_lang('bg', sys.stdout)
    
    def main2(filename):
        file = open(filename)
        ilis = []
        for line in file:
            ilis.append(line.strip())
        export = Export()
        export.export_ilis(ilis,  sys.stdout)
#    map = Map()
#    print map.conflicts
#    test2()
    main2('/home/hristina/Desktop/ili-ta')
