# -*- coding: utf-8 -*-

# Copyright 2006-2012 Borislav Rizov <bobyATdclDOTbasDOTbg>, Department of Computational Linguistic, Institute for Bulgarian Language – Bulgarian Academy of Sciences
# 
# This file is part of Hydra.
# 
# Hydra is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hydra is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Hydra.  If not, see <http://www.gnu.org/licenses/>.

import copy
class VM:
	Select, Open, Close, Edit, Save, Search, Text, Key = [1 << i for i in range(8)]
	All = -1
class VMessage:
	def __init__(self, data = None, type = VM.Select, senders = None):
		self.type = type
		self.data = data
		if senders:
			self.senders = senders
		else:
			self.senders = []
	def copy(self):
		senders = copy.copy(self.senders)
		return VMessage(self.data, self.type, senders)
class Context:
# Observer pattern
	#As central object
	def __init__(self):
		self.observers = dict()
	def attach(self, obs, interest = VM.Select,  att = 1):
		if(att):
			self.observers[obs] = interest
		else:
			if obs in self.observers:
				del self.observers[obs]
	def _obs_refresh(self, obs, message):
		if self.observers[obs] & message.type and obs not in message.senders:
			new_message = message.copy()
			new_message.type = self.observers[obs] & message.type
			obs.refresh(new_message)
		
	def notify(self, message):
		if self in self.observers:
			self._obs_refresh(self, message)
		message.senders.append(self)
		for obs in self.observers:
			self._obs_refresh(obs, message)
