# -*- coding: utf-8 -*-

# Copyright 2006-2012 Borislav Rizov <bobyATdclDOTbasDOTbg>, Department of Computational Linguistic, Institute for Bulgarian Language – Bulgarian Academy of Sciences
# 
# This file is part of Hydra.
# 
# Hydra is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hydra is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Hydra.  If not, see <http://www.gnu.org/licenses/>.

from linguistic_units import *
from _wn import *

class RelChars:
    Reflexive, Symetric, Transitive, NewCreational, Explicit, System = [1 << i for i in range(6)]

class Relation:
    reversed = False
    def __init__(self, info):
        info = list(info)
        info[RelationId.id1] = eval(info[RelationId.id1])
        info[RelationId.id2] = eval(info[RelationId.id2])
        self.info = info
    def __getitem__(self, item):
        return self.info[RelationId[item]]
    def is_(self, char):
        return bool(self.info[RelationId.chars] & char)
    def add(self, ling1, ling2):
        wn.insert(RelId.table, (ling1.ID(), ling2.ID(), self['id']))
        #print "add for relation"
    def _delete(self, ling1, ling2):
        wn.execute(u"delete from %s where id1 = %d and id2 = %d and rel = %s" % (RelId.table, ling1.ID(), ling2.ID(), self['id']))
        #print "delete for relation"
    def delete(self, ling1, ling2):
        self._delete(ling1, ling2)
        if self.is_(RelChars.Symetric):
            self._delete(ling2, ling1)
    def neighbours2(self, ling):
        return (wn.get_rel('~' + self.info[RelationId.name], ling) + wn.get_rel(self.info[RelationId.name], ling))
    def _neighbours(self, ling):
        return wn.get_rel('~' + self.info[RelationId.name], ling)
    def _rneighbours(self, ling):
        return wn.get_rel(self.info[RelationId.name], ling)
    def neighbours(self, ling):
        if self.is_(RelChars.Symetric):
            return self.neighbours2(ling)
        else:
            return self._neighbours(ling)
    def rneighbours(self, ling):
        if self.is_(RelChars.Symetric):
            return self.neighbours2(ling)
        else:
            return self._rneighbours(ling)
    def deletable(self):
        return True

class ReverseBase:
    def rneighbours(self, ling):
        return self.base().neighbours(self, ling)
    def neighbours(self, ling):
        return self.base().rneighbours(self, ling)
    def __getitem__(self, item):
        return self.info[RelationId.rnames[item]]
    def add(self, ling1, ling2):
        return self.base().add(self, ling2, ling1)
    def delete(self, ling1, ling2):
        self.base().delete(self, ling2, ling1)
    def base(self):
        return self.__class__.__bases__[1]
    def deletable(self):
        return not self.is_(RelChars.NewCreational)

class ReverseRelation(ReverseBase, Relation):
    reversed = True
    def __init__(self, info):
        Relation.__init__(self, info)
        
class LiteralRelation(Relation):
    def __init__(self, info):
        Relation.__init__(self, info)
    
class LNoteRelation(Relation):
    def __init__(self, info):
        Relation.__init__(self, info)

class SNoteRelation(Relation):
    def __init__(self, info):
        Relation.__init__(self, info)

class UsageRelation(Relation):
    def __init__(self, info):
        Relation.__init__(self, info)


class HypernymRelation(Relation):
    def __init__(self, info):
        Relation.__init__(self, info)

class HyponymRelation(ReverseBase, HypernymRelation):
    def __init__(self, info):
        HypernymRelation.__init__(self, info)
        
class ILIRelation(Relation):
    def __init__(self, info):
        Relation.__init__(self, info)
    def add(self, ling, ling2):
        ili = ling.to_string("ili")
        wn.execute(u"""insert ignore into %(rel)s
        select T1.id as id1, T2.id as id2,(select id from %(relation)s where name = 'ili') as rel 
        from %(main)s as T1 inner join %(main)s as T2 
        on T1.ili = '%(ili)s' and T2.ili = '%(ili)s' and T1.id != T2.id""" % {'rel': RelId.table, 'ili': ili, 'main': SynsetId.table, 'relation': RelationId.table })
        #print "add for relation"
    def delete(self, ling, ling2):
        query = u""" delete from %(rel)s where (%(rel)s.id1 = %(lid)s or %(rel)s.id2 = %(lid)s) and rel = (select id from %(relation)s where name = 'ili')""" \
            % {'lid': ling.ID(), 'rel': RelId.table, 'relation': RelationId.table}
        wn.execute(query)
    
class RelationsDict:
    def __init__(self):
        self.register = {'Literal':LiteralRelation, 'snote':SNoteRelation, 'lnote':LNoteRelation, 'usage':UsageRelation, 
        'hypernym':HypernymRelation, 'hyponym': HyponymRelation,
        'instance_hypernym':HypernymRelation, 'instance_hyponym': HyponymRelation,
        'ili':ILIRelation}
        self.relations = {}
        self.sys_relations = {}
        self.rev_relations = {}
        self.for_relations = {}
        rels = wn.get_all(RelationId.table)
        for rel in rels:
            rel_name = rel[RelationId.name]
            rrel_name = rel[RelationId.rname]
            if rrel_name != '':
                self.add_rel(rrel_name, self.create_relation(rrel_name, rel, True))
            self.add_rel(rel_name, self.create_relation(rel_name, rel, False))
        
##			if rrel_name in self.register:
##				relation = self.register[rrel_name](rel)
##				self.add_rel(rrel_name, relation)
    def create_relation(self, rel_name, rel, rev):
        if rel_name in self.register:
            relation = self.register[rel_name](rel)
        else:
            if rev:
                relation = ReverseRelation(rel)
            else:
                relation = Relation(rel)
        return relation
    def add_rel(self, rel_name, relation):
        if relation.is_(RelChars.System):
            self.sys_relations[rel_name] = relation
        else:
            self.relations[rel_name] = relation
