# -*- coding: utf-8 -*-

# Copyright 2006-2012 Borislav Rizov <bobyATdclDOTbasDOTbg>, Department of Computational Linguistic, Institute for Bulgarian Language – Bulgarian Academy of Sciences
# 
# This file is part of Hydra.
# 
# Hydra is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hydra is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Hydra.  If not, see <http://www.gnu.org/licenses/>.

from Tix import *
from wordnet import *
from observer_context import VMessage, VM
class TreeView(Frame):
    def create_relcombo(self):
        self.rel_combo = ComboBox(self, label = "Relation:", command = self.set_rel, options='listbox.height -1 label.anchor e')
        for rel in wn.relations:
##			print rel
            if wn.relations[rel].is_(RelChars.Transitive):
                self.rel_combo.append_history(rel)
        self.rel_combo.pick(0)
        self.rel_combo.pack(fill='x')
    def create_widgets(self):
        self.create_relcombo()
        
        # select
        self.selhyp_frame = Frame(self)
        self.selhyp_frame.pack(side = BOTTOM, anchor = W)
        self.selhyp = Select(self.selhyp_frame, label = "Select")
        self.selhyp.pack()
        # Info creation
        self.info = HList(self, width = 5, header = 1, bg = 'ivory', indent = 0, browsecmd = self.select, drawbranch = 0)
        self.info.header_create(0, itemtype = TEXT, text = 'Path')
        self.info.pack(side = LEFT,  fill = Y, anchor = W)
        # tree creation
        self.tree = Tree(self, bd = 0, bg = 'ivory', opencmd = self.opencmd, closecmd = self.closecmd, browsecmd = self.select,  options='hlist.columns 1 hlist.header 1')
        self.hlist = self.tree.hlist
        self.hlist.configure(background = 'ivory', selectforeground = 'black')
        self.hlist['yscrollcommand'] = self.sync
        self.tree.pack(fill = BOTH, expand = 1, anchor = W)
        self.hlist.header_create(0, itemtype = TEXT, text = 'Unit')
        
        self.style = DisplayStyle(TEXT, fg = 'black', bg = 'white', refwindow = self.hlist)
    

    def set_rel(self, rel):
        if rel != self.rel:
            self.rel = wn.relations[rel]
            self.refill(self.ling)
    def __init__(self, dict, parent = None):
        Frame.__init__(self, parent)
        self.rel = None
        self.ling = None
        self.dict = dict
        self.parent = parent
        self.create_widgets()
        self.lings = {}
    def opencmd(self, path):
        self.expand(self.rel, path)
    def closecmd(self, path): #Fixme: clear self.lings for offsprings
        self.hlist.delete_offsprings(path)
        self.info.delete_offsprings(path)
    def go_up2(self, rel, ling):
        hyps = rel.rneighbours(ling)
        if(hyps):
            for hyp in hyps:
                if hyp.ID() in self.lings:
##					print 'found'
                    self.expand(rel, self.lings[hyp.ID()])
                    return self.lings[ling.ID()]
            path = self.go_up(rel, hyps[0])
        else:
            self.clear()
            path = ''
        path = self.add_node(ling, `len(hyps)`, path)
        self.tree.setmode(path, 'close')
        return path
        
    def go_up(self, rel, ling,  level = 0):
        hyps = rel.neighbours(ling)
        if(hyps and level < 40):
            path = self.go_up(rel, hyps[0],  level + 1)
        else:
            path = ''
        path = self.add_node(ling, ` len(hyps)`, path)
        self.tree.setmode(path, 'close')
        return path
    def expand(self, rel, path):
        ling = wn.ling(long(self.hlist.info_data(path)))
        hyps = rel.rneighbours(ling)
        if(hyps):
            self.tree.setmode(path, 'close')
            for hyp in hyps:
                revhyps = rel.neighbours(hyp)
                p = self.add_node(hyp,  ` len(revhyps)`, path)
                hs = rel.rneighbours(hyp)
                if(hs):
                    self.tree.setmode(p, 'open')
                else:
                    self.tree.setmode(p, 'none')
        else:
            self.tree.setmode(path, 'none')

    def add_node(self, ling, info, path):
        p = self.hlist.add_child(path, itemtype = TEXT, text = ling.to_string(), data = ling.ID(), style = self.style);
        self.lings[ling.ID()] = p
        self.info.add_child(path, text = info)
        return p
    def clear(self):
        self.hlist.delete_all()
        self.info.delete_all()
        self.lings.clear()
    def sync(self, x, y):
        self.tree.vsb.set(x,y)
        self.info.yview('moveto', x)	
    def invokehyp(self, an_id, s):
        if s != '0':	
            ling = wn.ling(long(an_id))
            self.clear()
            path = self.go_up(self.rel, ling)
            revhyps = self.rel.neighbours(self.ling)
            path = self.add_node(self.ling, ` len(revhyps)`, path)
            self.expand(self.rel, path)
            self.sel(self.hlist, path)
            self.sel(self.info, path)
    def create_selhyp(self, rel, ling):
        self.selhyp.destroy()
        self.selhyp = Select(self.selhyp_frame, radio = 1, allowzero = 0)
        hyps = rel.neighbours(ling)
        if len(hyps) > 1:
            for i, hyp in enumerate(hyps):
                self.selhyp.add(unicode(hyp.ID()), text = i)
            self.selhyp.invoke(self.hlist.info_data(self.hlist.info_parent(self.lings[ling.ID()])))
            self.selhyp.config(command = self.invokehyp, label = "Select: ")
        self.selhyp.pack()
    def select(self, path):
        ling = wn.ling(long(self.hlist.info_data(path)))
        self.dict.refresh(VMessage(ling))
    def sel(self, hlist, p):	
        hlist.selection_clear()
        hlist.selection_set(p)
        self.style.config(bg = 'ivory')
        hlist.see(p)
    def set_ling(self, ling):
        p = self.lings[ling.ID()]
        
        #NB new
        self.hlist.entryconfigure(p, text = ling.to_string())
        #NB new
        
        self.sel(self.hlist, p)
        self.sel(self.info, p)
        self.create_selhyp(self.rel, ling)
        self.ling = ling
    def refill(self, ling):
        if(ling):
            self.clear()
            path = self.go_up(self.rel, ling)
            self.expand(self.rel, path)
    def refresh(self, message):
        ling = message.data
        if not ling:
            self.clear()
            self.ling = ling
        else:
            if(ling.ID() not in self.lings) or message.type & VM.Save:
                self.refill(ling)
            self.set_ling(ling)
    def memento(self):
        return self.ling
    def set_memento(self, memento):
        self.refresh(VMessage(memento))
