# -*- coding: utf-8 -*-

# Copyright 2006-2012 Borislav Rizov <bobyATdclDOTbasDOTbg>, Department of Computational Linguistic, Institute for Bulgarian Language – Bulgarian Academy of Sciences
# 
# This file is part of Hydra.
# 
# Hydra is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hydra is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Hydra.  If not, see <http://www.gnu.org/licenses/>.

from Tix import *
from observer_context import Context, VM
from ling_view import LingView, EditLingView
from synset_view import SynsetView
from tree_view import TreeView

class ViewHolder(Frame, Context):
	def __init__(self, parent, controller,  edit = True):
		Frame.__init__(self, parent)
		Context.__init__(self)
		self.parent = parent
		self.controller = controller
		self.components = []
		self.create_widgets(edit)
	def add_child(self, name, childClass, type):
		self.views.add(name, label = name)
		child = childClass(self.controller, self.views.page(name))
		child.pack(fill = BOTH, expand = 1)
		self.components.append(child)
		self.attach(child, type)
		return child
	def create_widgets(self,  edit):
		self.views = views = NoteBook(self)
		if edit:
			LingViewClass = EditLingView
		else:
			LingViewClass = LingView
		self.ling_view = self.add_child("main", LingViewClass, VM.Select|VM.Save|VM.Edit|VM.Open|VM.Close)
		self.tree_view = self.add_child("tree", TreeView, VM.Select|VM.Save)
		self.synset_view = self.add_child("synset", SynsetView, VM.Select|VM.Save)
		views.pack(fill = BOTH, expand = 1)
	def memento(self):
		return [child.memento() for child in self.components]
	def set_memento(self, memento):
		for i, child in enumerate(self.components):
			child.set_memento(memento[i])
	def refresh(self, message):
		self.notify(message)
	def enabled(self):
		return self.ling_view.enabled()
